// @sylefeb 2022-01-10
// MIT license, see LICENSE_MIT in Silice repo root
// https://github.com/sylefeb/Silice/

#include "config.h"
#include "sdcard.h"
#include "std.h"
#include "oled.h"
#include "display.h"
#include "printf.h"

#include "fat_io_lib/src/fat_filelib.h"

void callback()
{
  *LEDS = rdcycle();
}

void main()
{
  // install putchar handler for printf
  f_putchar = display_putchar;

  oled_init();
  oled_fullscreen();

  memset(display_framebuffer(),0x00,128*128);
  display_refresh();

  display_set_cursor(0,0);
  display_set_front_back_color(255,0);
  printf("init ... ");
  display_refresh();

  // init sdcard
  sdcard_init();
  // initialise File IO Library
  fl_init();
  // attach media access functions to library
  while (fl_attach_media(sdcard_readsector, sdcard_writesector) != FAT_INIT_OK) {
    // try again, we need this
  }
  printf("done.\n");
  display_set_cursor(0,0);
  // open the file
  FL_FILE *f = fl_fopen("/img.raw","rb");
  if (f == NULL) {
    printf("img.raw not found.\n");
    display_refresh();
  } else {
    display_refresh();
    // install callback
    sdcard_while_loading_callback = callback;
    // read pixels in framebuffer
    fl_fread(display_framebuffer(),1,128*128,f);
    display_refresh();
    // close
    fl_fclose(f);
  }

}
